/*
 * Test SRAM 1Mbyte (32bit databus) at CS3 for DIL/NetPC PNP/2110-3V
 * Author  : MHA@SSV <info@dilnetpc.com> <m.hasewinkel@web.de>
 * Date    : XX.XX.XXXX
 * Version : 0.00
 * License : GPL
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>

typedef unsigned char  u8;
typedef unsigned short u16;
typedef unsigned int   u32;

void *pMapmemory(off_t phy_addr, size_t phy_lenght);

#define MSC0  (*(pBASE  + 0x08 / 4))	/* MSC0 @ 0x48000008 */
#define MSC1  (*(pBASE  + 0x0C / 4))	/* MSC1 @ 0x4800000C */
#define MSC2  (*(pBASE  + 0x10 / 4))	/* MSC2 @ 0x48000010 */

#define SRAMSIZE 0x100000	/* 1 Mbyte */

int main (void)
{
    volatile u32 *pBASE, *pSRAM;
    u32 adr;

    // check user identity
    if (geteuid() != 0) {
	printf("No root access rights !\n");
	exit(1);
    }

    // map PXA255 register space at 0x48000000 / size 4096 bytes
    pBASE = pMapmemory(0x48000000, 0x00001000);
    if (pBASE == NULL) {
	printf("Can't map memory at 0x48000000 !\n");
	exit(1);
    }

    // map SRAM space at 0x0C000000 / size SRAMSIZE bytes
    pSRAM = pMapmemory(0x0C000000, SRAMSIZE);
    if (pSRAM == NULL) {
	printf("Can't map memory at 0x0C000000 !\n");
	exit(2);
    }

    MSC1 &= 0x0000ffff;		// MSC1 = 0x7ff1xxxx
    MSC1 |= 0x7ff10000;		// SRAM, 32bit, slow timing

    printf("Test SRAM 0x%08X bytes (%d)\n", SRAMSIZE, SRAMSIZE);
    printf("SRAM 1st test...\n");
    // 1st test
    for (adr=0; adr<(SRAMSIZE/4); adr++) {
	*(pSRAM+adr) = adr;
    }
    for (adr=0; adr<(SRAMSIZE/4); adr++) {
	if (*(pSRAM+adr) != adr) {
	    printf("SRAM FEHLER Adr. 0x%08X: soll 0x%08X != ist 0x%08X\n",
	    adr*4, adr, *(pSRAM+adr));
	    exit(3);
	}
    }

    printf("SRAM 2nd test...\n");
    // 2nd test
    for (adr=0; adr<(SRAMSIZE/4); adr++) {
	*(pSRAM+adr) = (adr^-1);
    }
    for (adr=0; adr<(SRAMSIZE/4); adr++) {
	if (*(pSRAM+adr) != (adr^-1)) {
	    printf("SRAM FEHLER Adr. 0x%08X: soll 0x%08X != ist 0x%08X\n",
	    adr*4, (adr^-1), *(pSRAM+adr));
	    exit(4);
	}
    }
    printf("SRAM OK...\n");

    return(0);
}

void *pMapmemory(off_t phy_addr, size_t phy_lenght)
{
#define MAP_PAGESIZE 4096UL

int iFd;
void *pMem;

    if ((phy_addr % MAP_PAGESIZE) != 0) {
	fprintf(stderr,"physical address not aligned on PAGESIZE boundary!\n");
	return(NULL);
    }

    if ((phy_lenght % MAP_PAGESIZE) != 0) {
	fprintf(stderr,"physical lenght not aligned on PAGESIZE boundary!\n");
	return(NULL);
    }

    // open mem device for read/write
    iFd = open("/dev/mem", O_RDWR | O_SYNC);
    if (iFd < 0) {
	fprintf(stderr,"open of /dev/mem fail !\n");
	return(NULL);
    }

    // get pointer to PNP2110 memory
    pMem = mmap(NULL,
		phy_lenght,
		(PROT_READ | PROT_WRITE),
		MAP_SHARED,
		iFd,
		phy_addr);

    if ((pMem == MAP_FAILED) || (pMem == NULL)) {
	fprintf(stderr,"mmap of /dev/mem fail !\n");
	return(NULL);
    }

    // close mem device
    if (close(iFd) != 0)
	fprintf(stderr,"close of /dev/mem fail !\n");

    return(pMem);
}
